package com.miya.manning.framework.request;

import com.google.gson.JsonObject;
import net.sf.json.JSONArray;
import net.sf.json.JSONObject;
import org.apache.commons.logging.Log;
import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.NameValuePair;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.utils.URLEncodedUtils;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.StringEntity;
import org.apache.http.entity.mime.MultipartEntity;
import org.apache.http.entity.mime.MultipartEntityBuilder;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.util.EntityUtils;
import org.apache.log4j.Logger;
import org.testng.collections.CollectionUtils;

import java.io.*;
import java.net.HttpURLConnection;
import java.net.URI;
import java.net.URL;
import java.util.*;
import java.util.Map.Entry;

/**
 * @ProjectName: MY_INTERFACE_TEST
 * @Package: com.miya.interfacetest.framework.request
 * @ClassName: HTTPPost
 * @Description: java类作用描述
 * @Author: 唐奇良
 * @CreateDate: 2019/1/21 下午5:39
 * @UpdateUser: 更新者
 * @UpdateDate: 2019/1/21 下午5:39
 */
public class HTTPPost {
    private static final Logger log = Logger.getLogger(HTTPPost.class);
    public static String httpPost(String url, JSONObject jsonParam) throws ClientProtocolException, IOException {
        HttpClientBuilder httpClientBuilder = HttpClientBuilder.create();
        CloseableHttpClient closeableHttpClient = httpClientBuilder.build();
        HttpPost httppost = new HttpPost(url);
        StringEntity entity = new StringEntity(jsonParam.toString(), "utf-8");
        entity.setContentEncoding("UTF-8");
        entity.setContentType("application/json");
        httppost.setEntity(entity);
        HttpResponse httpResponse;
        httpResponse = closeableHttpClient.execute(httppost);
        HttpEntity httpEntity = httpResponse.getEntity();
        String result = null;
        result = EntityUtils.toString(httpEntity, "UTF-8");
        closeableHttpClient.close();
        //System.out.print(result);
        return result;
    }

    public static String postMap(String url,Map<String,String> headerMap,Map<String, Object> contentMap) {
        String result = null;
        CloseableHttpClient httpClient = HttpClients.createDefault();
        HttpPost post = new HttpPost(url);

        CloseableHttpResponse response = null;
        try {
            Iterator headerIterator = headerMap.entrySet().iterator();          //循环增加header
            while(headerIterator.hasNext()){
                Entry<String,String> elem = (Entry<String, String>) headerIterator.next();
                post.addHeader(elem.getKey(),elem.getValue());
            }

            if (CollectionUtils.hasElements(contentMap)) {
                JSONObject jsonObject = JSONObject.fromObject(contentMap);
                StringEntity entity = new StringEntity(jsonObject.toString(), "utf-8");
                entity.setContentEncoding("UTF-8");
                entity.setContentType("application/json");

                post.setEntity(entity);
            }
            response = httpClient.execute(post);            //发送请求并接收返回数据
            if(response != null && response.getStatusLine().getStatusCode() == 200)
            {
                HttpEntity entity = response.getEntity();       //获取response的body部分
                result = EntityUtils.toString(entity);          //读取reponse的body部分并转化成字符串
            }
            return result;
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        } catch (ClientProtocolException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }finally {
            try {
                httpClient.close();
                if(response != null)
                {
                    response.close();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }

        }
        return null;
    }

    public static String postJson(String url, Map<String,String> headerMap, JsonObject contentJsonObj) {
        String result = null;
        CloseableHttpClient httpClient = HttpClients.createDefault();
        HttpPost post = new HttpPost(url);

        CloseableHttpResponse response = null;
        try {
            Iterator headerIterator = headerMap.entrySet().iterator();          //循环增加header
            while(headerIterator.hasNext()){
                Entry<String,String> elem = (Entry<String, String>) headerIterator.next();
                post.addHeader(elem.getKey(),elem.getValue());
            }

            if (contentJsonObj != null) {
                StringEntity entity = new StringEntity(contentJsonObj.toString(), "utf-8");
                entity.setContentEncoding("UTF-8");
                entity.setContentType("application/json");

                post.setEntity(entity);
            }
            response = httpClient.execute(post);            //发送请求并接收返回数据
            if(response != null && response.getStatusLine().getStatusCode() == 200)
            {
                HttpEntity entity = response.getEntity();       //获取response的body部分
                result = EntityUtils.toString(entity);          //读取reponse的body部分并转化成字符串
            }
            return result;
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        } catch (ClientProtocolException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }finally {
            try {
                httpClient.close();
                if(response != null)
                {
                    response.close();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }

        }
        return null;
    }

    public static String postMapOnlyUrl(String url,Map<String,String> headerMap,Map<String, String> contentMap) {
        String result = null;
        CloseableHttpClient httpClient = HttpClients.createDefault();
        HttpPost post = new HttpPost(url);

        CloseableHttpResponse response = null;
        try {
            Iterator headerIterator = headerMap.entrySet().iterator();          //循环增加header
            while(headerIterator.hasNext()){
                Entry<String,String> elem = (Entry<String, String>) headerIterator.next();
                post.addHeader(elem.getKey(),elem.getValue());
            }

            if (CollectionUtils.hasElements(contentMap)) {
                /*JSONObject jsonObject = JSONObject.fromObject(contentMap);
                StringEntity entity = new StringEntity(jsonObject.toString(), "utf-8");
                entity.setContentEncoding("UTF-8");
                entity.setContentType("application/json");
                post.setEntity(entity);*/
                List<NameValuePair> formparams = setHttpParams(contentMap);
                String param = URLEncodedUtils.format(formparams, "UTF-8");
                post.setURI(URI.create(url + "?" + param));
            }
            response = httpClient.execute(post);            //发送请求并接收返回数据
            if(response != null && response.getStatusLine().getStatusCode() == 200)
            {
                HttpEntity entity = response.getEntity();       //获取response的body部分
                result = EntityUtils.toString(entity);          //读取reponse的body部分并转化成字符串
            }
            return result;
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        } catch (ClientProtocolException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }finally {
            try {
                httpClient.close();
                if(response != null)
                {
                    response.close();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }

        }
        return null;
    }

    public static String httpPostForm (String url, Map<String, String> params) {
        URL u = null;
        HttpURLConnection con = null;
        // 构建请求参数
        StringBuffer sb = new StringBuffer();
        if (params != null) {
            for (Entry<String, String> e : params.entrySet()) {
                sb.append(e.getKey());
                sb.append("=");
                sb.append(e.getValue());
                sb.append("&");
            }
            sb.substring(0, sb.length() - 1);
        }
        System.out.println("send_url:" + url);
        System.out.println("send_data:" + sb.toString());
        // 尝试发送请求
        try {
            u = new URL(url);
            con = (HttpURLConnection) u.openConnection();
            //// POST 只能为大写，严格限制，post会不识别
            con.setRequestMethod("POST");
            con.setDoOutput(true);
            con.setDoInput(true);
            con.setUseCaches(false);
            con.setRequestProperty("Content-Type", "application/x-www-form-urlencoded");
            OutputStreamWriter osw = new OutputStreamWriter(con.getOutputStream(), "UTF-8");
            osw.write(sb.toString());
            osw.flush();
            osw.close();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (con != null) {
                con.disconnect();
            }
        }

        // 读取返回内容
        StringBuffer buffer = new StringBuffer();
        try {
            //一定要有返回值，否则无法把请求发送给server端。
            BufferedReader br = new BufferedReader(new InputStreamReader(con.getInputStream(), "UTF-8"));
            String temp;
            while ((temp = br.readLine()) != null) {
                buffer.append(temp);
                buffer.append("\n");
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

        return buffer.toString();
    }

    private static List<NameValuePair> setHttpParams(Map<String, String> paramMap) {
        List<NameValuePair> formparams = new ArrayList<NameValuePair>();
        Set<Entry<String, String>> set = paramMap.entrySet();
        for (Entry<String, String> entry : set) {
            formparams.add(new BasicNameValuePair(entry.getKey(), entry.getValue()));
        }
        return formparams;
    }

    public static String httpPost(String url, JSONObject jsonParam, Map<String, String> httpHeaderMap) throws ClientProtocolException, IOException {
        HttpClientBuilder httpClientBuilder = HttpClientBuilder.create();
        CloseableHttpClient closeableHttpClient = httpClientBuilder.build();
        HttpPost httppost = new HttpPost(url);

        //添加HTTPHeader
        Iterator headerIterator = httpHeaderMap.entrySet().iterator(); //循环增加header
        while (headerIterator.hasNext()) {
            Entry<String, String> elem = (Entry<String, String>) headerIterator.next();
            httppost.addHeader(elem.getKey(), elem.getValue());
        }
        httppost.addHeader("ContentType" , "application/json");

        StringEntity entity = new StringEntity(jsonParam.toString(), "utf-8");
        entity.setContentEncoding("UTF-8");
        entity.setContentType("application/json");
        httppost.setEntity(entity);
        HttpResponse httpResponse;
        httpResponse = closeableHttpClient.execute(httppost);
        HttpEntity httpEntity = httpResponse.getEntity();
        String result = null;
        result = EntityUtils.toString(httpEntity, "UTF-8");
        closeableHttpClient.close();
        //System.out.print(result);
        return result;
    }
    public static String httpuploadfile(String url, String file_path ,Map<String, String> httpHeaderMap) throws ClientProtocolException, IOException {
        String result = null;
        try{
            File file = new File(file_path);
            String filename = file.getName();
            FileInputStream in = null;
            InputStream is = null;
            if(null != file){
                try{
                    in = new FileInputStream(file);
                }catch (FileNotFoundException e){
                    log.error(e);
                }
                is = in;
            }
            HttpClientBuilder httpClientBuilder = HttpClientBuilder.create();
            CloseableHttpClient closeableHttpClient = httpClientBuilder.build();
            HttpPost httppost = new HttpPost(url);
        //添加HTTPHeader
            Iterator headerIterator = httpHeaderMap.entrySet().iterator(); //循环增加header
            while (headerIterator.hasNext()) {
                Entry<String, String> elem = (Entry<String, String>) headerIterator.next();
                httppost.addHeader(elem.getKey(), elem.getValue());
            }
            httppost.addHeader("Content-Disposition","form_data; name=\"file\"; filename=\"" + filename + "\"");
            MultipartEntityBuilder builder = MultipartEntityBuilder.create();
            builder.addBinaryBody("file", is , ContentType.MULTIPART_FORM_DATA , filename);
            HttpEntity entity = builder.build();
            httppost.setEntity(entity);
            HttpResponse httpResponse;
            httpResponse = closeableHttpClient.execute(httppost);
            HttpEntity httpEntity = httpResponse.getEntity();
            result = EntityUtils.toString(httpEntity, "UTF-8");
            closeableHttpClient.close();
            //System.out.print(result);
            if(null != is){
                is.close();
            }
            if(null != in){
                in.close();
            }
            if(null != closeableHttpClient){
                closeableHttpClient.close();
            }
        }catch (Exception e){
            log.error(e);
        }
        return result;
    }

    public static String httpPost(String url, JSONArray jsonParam, Map<String, String> httpHeaderMap) throws ClientProtocolException, IOException {
        HttpClientBuilder httpClientBuilder = HttpClientBuilder.create();
        CloseableHttpClient closeableHttpClient = httpClientBuilder.build();
        HttpPost httppost = new HttpPost(url);

        //添加HTTPHeader
        Iterator headerIterator = httpHeaderMap.entrySet().iterator(); //循环增加header
        while (headerIterator.hasNext()) {
            Entry<String, String> elem = (Entry<String, String>) headerIterator.next();
            httppost.addHeader(elem.getKey(), elem.getValue());
        }

        StringEntity entity = new StringEntity(jsonParam.toString(), "utf-8");
        entity.setContentEncoding("UTF-8");
        entity.setContentType("application/json");
        httppost.setEntity(entity);
        HttpResponse httpResponse;
        httpResponse = closeableHttpClient.execute(httppost);
        HttpEntity httpEntity = httpResponse.getEntity();
        String result = null;
        result = EntityUtils.toString(httpEntity, "UTF-8");
        closeableHttpClient.close();
        //System.out.print(result);
        return result;
    }
}