package com.miya.manning.framework.util;

/**
 * @ProjectName: MY_INTERFACE_TEST
 * @Package: com.miya.manning.framework.util
 * @ClassName: GPSUtil
 * @Description: 定位距离计算工具类
 * @Author: 梁振振
 * @CreateDate: 2020/8/11 下午18：02
 * @UpdateUser:
 * @UpdateDate:
 */

public class GPSUtil {

    private static final double EARTH_RADIUS = 6371000;// 赤道半径(单位m)

    /**
     * 转化为弧度(rad)
     */
    private static double rad(double d) {
        return d * Math.PI / 180.0;
    }

    /**
     * 基于googleMap中的算法得到两经纬度之间的距离,计算精度与谷歌地图的距离精度差不多，相差范围在0.2米以下（单位m）
     *
     * @param lon1 第一点的精度
     * @param lat1 第一点的纬度
     * @param lon2 第二点的精度
     * @param lat2 第二点的纬度
     * @return 返回的距离，单位m
     */
    public static double GetDistance(double lon1, double lat1, double lon2, double lat2) {
        double radLat1 = rad(lat1);
        double radLat2 = rad(lat2);
        double a = radLat1 - radLat2;
        double b = rad(lon1) - rad(lon2);
        double s = 2 * Math.asin(Math.sqrt(
                Math.pow(Math.sin(a / 2), 2) + Math.cos(radLat1) * Math.cos(radLat2) * Math.pow(Math.sin(b / 2), 2)));
        s = s * EARTH_RADIUS;
        // s = Math.round(s * 10000) / 10000;
        return s;

    }

    public static double GetDistance(Object o_lon1, Object o_lat1, Object o_lon2, Object o_lat2) {
        double lon1 = Double.valueOf(o_lon1.toString());
        double lat1 = Double.valueOf(o_lat1.toString());
        double lon2 = Double.valueOf(o_lon2.toString());
        double lat2 = Double.valueOf(o_lat2.toString());
        double radLat1 = rad(lat1);
        double radLat2 = rad(lat2);
        double a = radLat1 - radLat2;
        double b = rad(lon1) - rad(lon2);
        double s = 2 * Math.asin(Math.sqrt(
                Math.pow(Math.sin(a / 2), 2) + Math.cos(radLat1) * Math.cos(radLat2) * Math.pow(Math.sin(b / 2), 2)));
        s = s * EARTH_RADIUS;
        // s = Math.round(s * 10000) / 10000;
        return s;

    }

    /*
    * @description: saas距离，距离小于1km，显示XXm，不保留小数； 距离大于1km，显示XX.Xkm, 保留一位小数
    * @param distance: 传入的距离单位为米
    * @return：单位：米 如果大于1000，则十位数和个位数为0
    * */
    public static double GetSaasDistance(double distance) {
        if (distance >= 1000) {
            //如果距离大于1000m，则saas会显示km为单位的距离，保留一位小数，即返回结果为XX00.0m
            return (Math.round(distance/100) * 0.1d) * 1000;
        } else {
            //如果距离小于1000m，则saas会显示m为单位的距离，不保留小数，即返回结果为XXX.0m
            return Math.round(distance);
        }
    }

    public static double GetSaasDistance(Object o_lon1, Object o_lat1, Object o_lon2, Object o_lat2){
        return GetSaasDistance(GetDistance(o_lon1, o_lat1, o_lon2, o_lat2));
    }


}
