package com.miya.manning.framework.util;

import com.miya.manning.enums.DataRefTypeEnum;
import com.miya.manning.framework.util.dbutil.MysqlUtil;
import com.miya.manning.test.flow.order.Express;
import com.miya.manning.vo.conditions.Condition;
import com.miya.manning.vo.scripts.Script;
import com.miya.manning.vo.testcase.TestCase;
import org.apache.log4j.Logger;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellStyle;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.xssf.usermodel.XSSFCell;
import org.apache.poi.xssf.usermodel.XSSFRow;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;

import java.math.BigInteger;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.stream.Collectors;

public class TestCaseUtil {




    public static Logger logger = Logger.getLogger(TestCaseUtil.class);;
    /**
     * 获取一组用例的主要信息,如接口名,方法类型等
     * @return 返回一个List<TestCase>对象
     */

    public static Object[][] GetMain(String path) throws RuntimeException {
        List<TestCase> testCases = new ArrayList<>();
        /*ResourceBundle bundle = ResourceBundle.getBundle("task");
        String case_path = bundle.getString("testcase");*/
        XSSFSheet sheet = ExcelUtil.ExcelData(path, "sheet1");
        for(int j = 1 ; j< sheet.getPhysicalNumberOfRows();j++){
            XSSFRow eachrow = sheet.getRow(j);
            if(!ExcelUtil.IsRowEmpty(eachrow)){

                //如果该行不是空,才建立Testcase对象
                TestCase ts = new TestCase();

                //按照用例约定格式,依次给Testcase对象各属性赋值
                ts.setNo(Integer.parseInt(eachrow.getCell(0).getRawValue() == null ? "0" : eachrow.getCell(0).toString() ));//设置用例组号
                ts.setTest_point(eachrow.getCell(1, Row.RETURN_NULL_AND_BLANK) == null ? "" : eachrow.getCell(1).toString());//设置测试点
                ts.setStep_desc(eachrow.getCell(2, Row.RETURN_NULL_AND_BLANK) == null ? "" : eachrow.getCell(2).toString());//设置步骤描述
                ts.setInterface_name(eachrow.getCell(3, Row.RETURN_NULL_AND_BLANK).toString());//设置url
                ts.setMethod_type(eachrow.getCell(4, Row.RETURN_NULL_AND_BLANK).toString());//设置方法类型
                XSSFCell cell = eachrow.getCell(5,Row.RETURN_NULL_AND_BLANK);
                if(cell.getHyperlink() != null ){//如果是超链接
                    ts.setIs_uploadfile(true);
                    //获取文件名即可
                    String path_name = cell.getHyperlink().getAddress();
                    String file_name = path.substring(0,path.lastIndexOf("\\") + 1) + path_name;
                    ts.setPath(file_name);

                }else{
                    ts.setIs_uploadfile(false);
                    ts.setPath(eachrow.getCell(5, Row.RETURN_NULL_AND_BLANK).toString());//设置引用数据路径
                }
                ts.setExpect(eachrow.getCell(6, Row.RETURN_NULL_AND_BLANK) == null ? "" : eachrow.getCell(6).toString());//设置期望结果
                ts.setCheck_sql(eachrow.getCell(7, Row.RETURN_NULL_AND_BLANK) == null ? "" : eachrow.getCell(7).toString());//设置检查语句
                ts.setPre_sql(eachrow.getCell(8, Row.RETURN_NULL_AND_BLANK) == null ? "" : eachrow.getCell(8).toString());//设置前置语句
                ts.setPlatform(eachrow.getCell(9, Row.RETURN_NULL_AND_BLANK) == null ? "C" : eachrow.getCell(9).toString());//设置是C端接口还是B端接口,默认C端
                if(eachrow.getCell(10, Row.RETURN_NULL_AND_BLANK) == null || eachrow.getCell(10).toString().isEmpty()){
                    ts.setIs_excute("Y");
                }else{
                    ts.setIs_excute(eachrow.getCell(10).toString());
                }
                ts.setDatabasename(eachrow.getCell(11, Row.RETURN_NULL_AND_BLANK) == null ? "" : eachrow.getCell(11).toString());//设置数据库名称
                ts.setScript_name(eachrow.getCell(12, Row.RETURN_NULL_AND_BLANK)== null ? "" : eachrow.getCell(12).toString());//设置脚本名称
                ts.setCondition_name(eachrow.getCell(13, Row.RETURN_NULL_AND_BLANK)== null ? "" : eachrow.getCell(13).toString());//设置脚本名称
                testCases.add(ts);
            }
        }
        return GetData(path,testCases);
    }

    public static List<Script> GetScripts(String script_path) throws RuntimeException {
        List<Script> Scripts = new ArrayList<>();
        /*ResourceBundle bundle = ResourceBundle.getBundle("task");
        String case_path = bundle.getString("testcase");*/
        XSSFSheet sheet = ExcelUtil.ExcelData(script_path, "sheet1");
        for(int j = 1 ; j< sheet.getPhysicalNumberOfRows();j++){
            XSSFRow eachrow = sheet.getRow(j);
            if(!ExcelUtil.IsRowEmpty(eachrow)){
                for(int p = 0 ; p< eachrow.getLastCellNum(); p++){
                    eachrow.getCell(p).setCellType(Cell.CELL_TYPE_STRING);//每个字段都处理成纯文本
                }
                //如果该行不是空,才建立Testcase对象
                Script script = new Script();

                //按照用例约定格式,依次给Testcase对象各属性赋值
                script.setScript_name(eachrow.getCell(0) == null ? "" : eachrow.getCell(0).toString());//设置脚本名称
                script.setSrcipt_type(eachrow.getCell(1, Row.RETURN_NULL_AND_BLANK) == null ? "" : eachrow.getCell(1).toString());//设置脚本类型
                script.setScript_param(eachrow.getCell(2, Row.RETURN_NULL_AND_BLANK) == null ? "" : eachrow.getCell(2).toString());//设置脚本参数
                script.setScript_usage(eachrow.getCell(3, Row.RETURN_NULL_AND_BLANK) == null ? "" : eachrow.getCell(3).toString());//设置脚本说明
                Scripts.add(script);
            }
        }
        return Scripts;
    }


    public static List<Condition> GetConditions(String condition_path) throws RuntimeException {
        List<Condition> Conditions = new ArrayList<>();
        /*ResourceBundle bundle = ResourceBundle.getBundle("task");
        String case_path = bundle.getString("testcase");*/
        XSSFSheet sheet = ExcelUtil.ExcelData(condition_path, "sheet1");
        for(int j = 1 ; j< sheet.getPhysicalNumberOfRows();j++){
            XSSFRow eachrow = sheet.getRow(j);
            if(!ExcelUtil.IsRowEmpty(eachrow)){
                for(int p = 0 ; p< eachrow.getLastCellNum(); p++){
                    eachrow.getCell(p).setCellType(Cell.CELL_TYPE_STRING);//每个字段都处理成纯文本
                }
                //如果该行不是空,才建立Testcase对象
                Condition condition = new Condition();

                //按照用例约定格式,依次给Testcase对象各属性赋值

                condition.setCondition_name(eachrow.getCell(0) == null ? "" : eachrow.getCell(0).toString());//设置控制器名称
                condition.setCondition_type(eachrow.getCell(1, Row.RETURN_NULL_AND_BLANK) == null ? "" : eachrow.getCell(1).toString());//设置控制器类型
                condition.setCondition_param(eachrow.getCell(2, Row.RETURN_NULL_AND_BLANK) == null ? "" : eachrow.getCell(2).toString());//设置控制器参数
                condition.setCondition_expect(eachrow.getCell(3, Row.RETURN_NULL_AND_BLANK) == null ? "" : eachrow.getCell(3).toString());//设置控制器比较值
                condition.setCondition_usage(eachrow.getCell(4, Row.RETURN_NULL_AND_BLANK) == null ? "" : eachrow.getCell(4).toString());//设置控制器说明
                condition.setCondition_compare_type(eachrow.getCell(5,Row.RETURN_NULL_AND_BLANK) == null ? "" : eachrow.getCell(5).toString());//设置控制器比较类型
                Conditions.add(condition);
            }
        }
        return Conditions;
    }

    /**
     *
     * @param testCases 接收TestCase对象列表,组装好params参数后返回
     * @return
     */
    public static Object[][] GetData(String path ,List<TestCase> testCases){
        /*ResourceBundle bundle = ResourceBundle.getBundle("task");
        String path = bundle.getString("testcase");//用来组装绝对路径*/
        List<TestCase> excute_testCases = testCases.stream().filter(x->"Y".equals(x.getIs_excute())).collect(Collectors.toList());
        List<Object[]> tss = new ArrayList<>();
        for(TestCase testCase:excute_testCases){
            if(!testCase.getPath().isEmpty() && !testCase.isIs_uploadfile()){
                String data_path = path.substring(0,path.lastIndexOf("\\")+1) + testCase.getPath().substring(0, testCase.getPath().lastIndexOf("|"));
                String nums = testCase.getPath().substring(testCase.getPath().lastIndexOf("|")+1 );
                XSSFWorkbook sheets = ExcelUtil.ExcelData(data_path);
                if("get".equalsIgnoreCase(testCase.getMethod_type())){
                    if(nums.split(",").length >1 ){
                        logger.error("get方法不能对应多组参数");
                        continue;
                    }
                    String data = ExcelUtil.getdata(sheets);
                    testCase.setParams(data);
                }
                else if("post".equalsIgnoreCase(testCase.getMethod_type())){
                    String data = ExcelUtil.postdata(sheets, sheets.getSheetName(0), DataRefTypeEnum.Type_normal.getTypeId(), nums.split(","));
                    //System.out.println(data);
                    testCase.setParams(data);
                }

            }
            Object[] obj = new Object[]{testCase};
            tss.add(obj);

        }

        Object[][] objs  = new Object[tss.size()][];
        tss.toArray(objs);
        return objs;
    }

    public static boolean check(String expect ,Map<String,Integer> int_var_map , Map<String,String> string_var_map,  Map<String,String> res_map){
        //期望值可能也有变量,需要替换
        boolean flag = true;
        if(!"".equals(expect)){
            logger.info("替换前的期望值为:" +  expect);
            expect = ParamsBuilt.params_replace(expect,int_var_map ,string_var_map ,res_map );
            logger.info("替换后的期望值为:" +  expect);
            String[] split = expect.split(";");
            //先按;分组，多个期望值一起校验
            for(String each_expect : split){
                if(each_expect.matches(".*<>.*")){
                    String[] split1 = each_expect.split("<>");
                    logger.info(split1[0] + "采用不等校验");
                    if(!res_map.containsKey(split1[0])){
                        logger.error("响应中未获取到对应的变量:" + split1[0]);
                        return false;
                    }
                    else{
                        String true_value = res_map.get(split1[0]).toLowerCase();
                        String check_string = split1[1].toLowerCase();
                        flag = true_value.matches(check_string);
                        if(flag){
                            logger.info("校验模式:不等校验,根据条件:" + check_string + "匹配实际值" + true_value + ",校验结果:失败");
                            flag = false;
                        }else{
                            logger.info("校验模式:不等校验,根据条件:" + check_string + "匹配实际值" + true_value + ",校验结果:成功");
                        }
                    }
                }
                if(each_expect.matches(".*=.*")){
                    String[] split1 = each_expect.split("=");
                    logger.info(split1[0] + "采用相等校验");
                    if(!res_map.containsKey(split1[0])){
                        logger.error("响应中未获取到对应的变量:" + split1[0]);
                        return false;
                    }
                    else{
                        String true_value = res_map.get(split1[0]).toLowerCase();
                        String check_string = split1[1].toLowerCase();
                        flag = true_value.matches(check_string);
                        if(flag){
                            logger.info("校验模式:相等校验,根据条件:" + check_string + "匹配实际值" + true_value + ",校验结果:成功");

                        }else{
                            logger.info("校验模式:相等校验,根据条件:" + check_string + "匹配实际值" + true_value + ",校验结果:失败");
                            flag = false;
                        }
                    }
                }
                /*String[] split1 = each_expect.split("=");
                if (split1[1].startsWith("*") && split1[1].endsWith("*")){
                    String check_string = split1[1].substring(1,split1[1].length()-2); //*abc*
                    logger.info(split1[0] + "采用通配符校验");
                    if(!res_map.containsKey(split1[0])) {
                        logger.error("响应中未获取到对应的变量:" + split1[0]);
                        return false;
                    }if(!res_map.get(split1[0]).toLowerCase().contains(check_string.toLowerCase())){
                        logger.error( split1[0] + "的期望值:"+ "不匹配！");
                        return false;
                    }else{
                        logger.info(split1[0]+ "的期望值校验成功");
                    }
                }
                else{
                    if(!res_map.containsKey(split1[0])){
                        logger.error("响应中未获取到对应的变量:" +split1[0] );
                        return false;
                    }if (!res_map.get(split1[0]).equalsIgnoreCase(split1[1])){
                        logger.error( split1[0] + "的期望值:"+ "不匹配！");
                        return false;
                    }else{
                        logger.info(split1[0]+ "的期望值校验成功");
                    }
                }*/

            }
        }
        return flag;
    }

    public static void deal_pre_sql(String pre_sql , Map<String,String> string_var_map , Map<String, Integer> int_var_map, Map<String,String> res_map , String databasename) throws Exception{
        if(!"".equals(pre_sql)){
            String[] subs = pre_sql.split(";");
            for(String each: subs){
                //按|分割变量部分和sql部分
                String[] sub  = each.split("\\|");
                if(sub.length>1){
                    String var = sub[0];
                    String exec_sql = sub[1];
                    logger.info("替换前的sql语句为:" + exec_sql);
                    String new_exec_sql = ParamsBuilt.sql_replace(exec_sql, int_var_map, string_var_map, res_map);
                    logger.info("替换后的sql语句为:" + new_exec_sql);

                    if(!"".equals(new_exec_sql)){
                        if(new_exec_sql.toLowerCase().startsWith("select")){
                            if(!databasename.isEmpty()){
                                logger.info("切换至" + databasename + "进行查询");
                            }
                            List<Map<String,Object>> query_data = MysqlUtil.ExecSelectSQL(new_exec_sql,databasename);

                            if(query_data.isEmpty()){
                                logger.info("前置语句未查询到任何数据!请检查sql");
                            }
                            else if(query_data.size()>1){
                                logger.info("前置语句查到多条数据!请检查sql");
                                //此时不替换变量
                            }
                            else{
                                for(Map.Entry<String,Object> entry:query_data.get(0).entrySet()){
                                    if(entry.getValue() instanceof BigInteger){
                                        int_var_map.put(var,(Integer.parseInt(entry.getValue().toString())));
                                    }
                                    else if(entry.getValue() instanceof Long){
                                        Number value = (Number) entry.getValue();
                                        int_var_map.put(var,value.intValue());
                                    }
                                    else if(entry.getValue() instanceof Integer){
                                        int_var_map.put(var,(Integer)entry.getValue());
                                    }
                                    else{
                                        string_var_map.put(var, (String) entry.getValue());
                                    }
                                }
                            }
                        }
                    }
                }
                else{
                    String exec_sql = sub[0];
                    logger.info("替换前的sql语句为:" + exec_sql);
                    String new_exec_sql = ParamsBuilt.sql_replace(exec_sql, int_var_map, string_var_map, res_map);
                    logger.info("替换前的sql语句为:" + new_exec_sql);
                    int rows = MysqlUtil.ExecuteSingleUpdateSQL(new_exec_sql,databasename);
                    if(rows==0){
                        logger.error("前置语句执行失败,请检查!");
                        throw new Exception("前置语句执行失败,请检查!");
                        //前置条件失败后,如果是dql语句,不影响后续用例执行,如果还是ddl语气,该组用例不执行
                    }
                }
            }
        }
    }

    /***
     * 对检查语句做变量替换.并检查和设定值是否一致
     * @param check_sql 检查sql语句
     * @param string_var_map string字符变量集
     * @param int_var_map int字符变量集
     * @param res_map 应答结果变量集
     * @return 返回比对结果 一致返回true 不一致返回false
     */
    public static boolean do_check_sql(String check_sql , Map<String,String> string_var_map , Map<String, Integer> int_var_map, Map<String,String> res_map,String databasename) throws Exception {
        boolean check_flag = true;
        if(!"".equals(check_sql)){
            String[] subs = check_sql.split(";");
            for(String each: subs){
                //按|分割校验值部分和sql部分
                String[] sub  = each.split("\\|");
                if(sub.length>1){
                    String var = sub[0];
                    String exec_sql = sub[1];
                    logger.info("替换前的sql语句为:" + exec_sql);
                    String new_exec_sql = ParamsBuilt.sql_replace(exec_sql, int_var_map, string_var_map, res_map);
                    logger.info("替换后的sql语句为:" + new_exec_sql);
                    List<Map<String,Object>> query_data = MysqlUtil.ExecSelectSQL(new_exec_sql,databasename);
                    if(query_data.isEmpty()){
                        logger.error("检查语句未查询到任何数据!请检查sql");
                        throw new Exception("检查语句未查询到任何数据!请检查sql");
                    }
                    else if(query_data.size()>1){
                        logger.error("检查语句查到多条数据!请检查sql");
                        throw new Exception("检查语句未查询到任何数据!请检查sql");
                        //此时不替换变量
                    }
                    else{
                        for(Map.Entry<String,Object>  entry : query_data.get(0).entrySet()){
                            logger.info("检查语句查询到的结果为:" +  entry.getValue().toString());
                            if(var.equals((entry.getValue().toString()))){
                                logger.info("检查结果:"+ var + "与查询结果比对通过" );
                            }else {
                                logger.error("检查结果:" + var + "与查询结果比对失败!");
                                check_flag = false;
                            }
                        }

                    }
                }
            }
        }
        //检查语句为空,不需要检查,返回true
        return check_flag;
    }


}
