package com.miya.manning.framework.util;

import com.miya.manning.test.flow.order.Express;
import com.miya.manning.vo.conditions.Condition;
import com.miya.manning.vo.scripts.Script;
import com.miya.manning.vo.testcase.TestCase;
import org.apache.log4j.Logger;
import org.testng.Assert;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

public class TestUtil {
    static Logger logger = Logger.getLogger(TestUtil.class);

    public static void doTest(TestCase testCase , Map<String,String> string_var_map, Map<String,Integer> int_var_map,
                              Map<String,String> res_map , List<Script> Scripts, List<Condition> Conditions,
                              String url_start) throws Exception{
        Map<String,String> one_res_map = new HashMap<>();//用来保存每个用例执行的变量结果集本地副本
        //前置语句最先执行
        String pre_sql = testCase.getPre_sql();
        String databasename = testCase.getDatabasename();
        //1.处理前置语句,赋值给变量或者直接执行
        TestCaseUtil.deal_pre_sql(pre_sql, string_var_map, int_var_map, res_map , databasename);
        //检查是否有控制器,判断是否执行该用例
        String condition_name = testCase.getCondition_name();
        if(!condition_name.isEmpty()){
            List<Condition> execConditionList = Conditions.stream().filter(x -> x.getCondition_name().equals(testCase.getCondition_name())).collect(Collectors.toList());
            boolean flag = ToolUtil.ExecuteCondition(execConditionList, Scripts ,testCase ,string_var_map, int_var_map, res_map);
            if(flag){
                //控制器返回结果为true时,说明已经执行过用例了或者无需执行用例,这里就不用再执行一次
                logger.info("取消当前组用例执行,用例名称:" + testCase.getStep_desc() );
                return;
            }
        }
        String script_name = testCase.getScript_name();
        if(!script_name.isEmpty()){
            List<Script> execScriptList = Scripts.stream().filter(x -> x.getScript_name().equals(testCase.getScript_name())).collect(Collectors.toList());
            ToolUtil.ExecuteScript(execScriptList);
        }

        //用例执行部分
        //2.设置请求头
        Map<String, String> httpheadermap = new HashMap<>();
        if("C".equals(testCase.getPlatform())){
            httpheadermap  = HttpHeaderBuilt.httpHeadBuiltManningsC(string_var_map,int_var_map);
        }
        if("B".equals(testCase.getPlatform())){
            httpheadermap = HttpHeaderBuilt.httpHeadBuiltManningsB(res_map);
        }

        logger.info("以下是请求头信息:");
        for(Map.Entry<String,String> entry: httpheadermap.entrySet()){
            logger.info(entry.getKey() + ":" + entry.getValue());
        }
        //4.调用前替换params中可能存在的变量
        String params = testCase.getParams();
        logger.info("替换前的参数为:" + params);
        String new_params = ParamsBuilt.params_replace(params, int_var_map, string_var_map ,res_map);
        logger.info("替换后的参数为:" + new_params);
        //5.拼接url,调用并获取接口返回
        StringBuffer url = new StringBuffer(url_start);
        StringBuilder builder = new StringBuilder();
        url.append(testCase.getInterface_name());//先拼接一部分
        String new_url = ParamsBuilt.params_replace(url.toString(), int_var_map, string_var_map, res_map);//替换url中可能存在的变量
        StringBuffer new_url_bf = new StringBuffer(new_url);
        RequestUtil.doRequest(testCase, new_params, new_url_bf, httpheadermap, logger, builder);
        //6.收集响应信息，并按规则添加至map中
        one_res_map = JsonTransfer.AnalysisJson(builder.toString(),one_res_map);
        //7.执行检查语句,根据本次返回的接口内容来校验,和期望值相同
        String check_sql = testCase.getCheck_sql();
        Assert.assertEquals(TestCaseUtil.do_check_sql(check_sql,string_var_map,int_var_map,one_res_map,databasename), true);
        //8.校验期望值,校验本次接口返回的内容,用one_res_map来校验
        String expect = testCase.getExpect();
        Assert.assertEquals(TestCaseUtil.check(expect, int_var_map,string_var_map,one_res_map) , true);
        //本次请求变量添加到总的变量集中
        res_map.putAll(one_res_map);
    }
}
